@@ -3,8 +3,7 @@ from celery.backends.database import Backend
 from celery.utils import gen_unique_id
 from celery.task import PeriodicTask
 from celery import registry
-from celery.models import PeriodicTaskMeta
-from datetime import datetime, timedelta
+from datetime import timedelta
 
 
 class SomeClass(object):
@@ -25,13 +24,6 @@ registry.tasks.register(MyPeriodicTask)
 class TestDatabaseBackend(unittest.TestCase):
 
     def test_run_periodic_tasks(self):
-        #obj, created = PeriodicTaskMeta.objects.get_or_create(
-        #                    name=MyPeriodicTask.name,
-        #                    defaults={"last_run_at": datetime.now() -
-        #                        timedelta(days=-4)})
-        #if not created:
-        #    obj.last_run_at = datetime.now() - timedelta(days=4)
-        #    obj.save()
         b = Backend()
         b.run_periodic_tasks()
 
