@@ -1,3 +1,4 @@
+from __future__ import with_statement
 import sys
 import unittest
 
@@ -6,8 +7,9 @@ class TestAAPickle(unittest.TestCase):
 
     def test_no_cpickle(self):
         from celery.tests.utils import mask_modules
-        del(sys.modules["celery.serialization"])
+        prev = sys.modules.pop("celery.serialization")
         with mask_modules("cPickle"):
             from celery.serialization import pickle
             import pickle as orig_pickle
             self.assertTrue(pickle.dumps is orig_pickle.dumps)
+        sys.modules["celery.serialization"] = prev
