@@ -69,5 +69,5 @@ latex_documents = [
    ur'Ask Solem', 'manual'),
 ]
 
-html_theme = "nature"
+html_theme = "ADCTheme"
 html_theme_path = ["_theme"]
